//
//  dataManagement-msync.swift
//  cviko-tv
//
//  Created by Martin Hruby on 18.04.18.
//  Copyright © 2018 Martin Hruby. All rights reserved.
//

import Foundation
import CoreData
import CloudKit


//
extension DataMAN {
    //
    func processSave(dict: [CKRecord:SyncedNote], finalCKs: [CKRecord]?) {
        //
        guard let _finalCKs = finalCKs else { return }
        
        //
        for fck in _finalCKs {
            //
            if let _rec = dict[fck] {
                //
                _rec.wasSyncedToCloud(withCKRecord: fck)
            }
        }
    
        //
        localSAVE();
    }
    
    //
    func processUpdates(toSave: [CKRecord], toDel: [CKRecordID]) {
        //
        toSave.forEach {
            //
            SyncedNote.updateFromCK(ck: $0)
        }
        
        //
        toDel.forEach {
            //
            SyncedNote.updateFromCK(deleted: $0)
        }
        
        //
        localSAVE();
    }
    
    //
    func fetchAndSyncModifiedToCloud() {
        //
        guard
            let _modifs = SyncedNote.allRecords(modifOnly: true),
            _modifs.isEmpty == false
            else { return }
        
        //
        _syncSavesToCloud(_modifs)
    }
        
    //
    func _syncSavesToCloud(_ notes: [SyncedNote]) {
        //
        if __syncLocked { return }
        
        //
        __syncLocked = true
        
        //
        print("Sync saves: \(notes.count)")
        
        //
        let toSave = notes.yourCKs()
        let toSaveCKs = [CKRecord](toSave.keys)
        
        //
        let _op = CKModifyRecordsOperation(recordsToSave: toSaveCKs,
                                           recordIDsToDelete: [])
        
        //
        _op.modifyRecordsCompletionBlock = { saved, _, error in
            //
            self.__syncLocked = false
            
            //
            if let _error = error {
                //
                print("Save to CK: \(_error.localizedDescription)")
            } else {
                //
                print("Save to CK: cool!")
            }
            
            //
            self.processSave(dict: toSave, finalCKs: saved)
        }
        
        //
        ckDB.add(_op)
    }
    
    //
    func syncDeletes(recIds: [CKRecordID]) {
        //
        print("Sync deletes: \(recIds.count)")
        
        //
        let _op = CKModifyRecordsOperation(recordsToSave: [],
                                           recordIDsToDelete: recIds)
        
        //
        _op.modifyRecordsCompletionBlock = { saved, _, error in
            //
            if let _error = error {
                //
                print("DEL to CK: \(_error.localizedDescription)")
            } else {
                //
                print("DEL to CK: cool!")
            }
        }
        
        //
        ckDB.add(_op)
    }
    
    //
    func doDiffFetches(whenDone: @escaping ()->()) {
        //
        print("DFetch req")
        //
        if __diffFetchInProgress { return }
        
        print("DFetch prep")
        // pole pro ulozeni ziskanych CKRecord
        var recordsToSave = [CKRecord]()
        var recordIDsToDelete = [CKRecordID]()
        
        // options pro jednotlive RecordZone
        var optsDict = [CKRecordZoneID : CKFetchRecordZoneChangesOptions]()
        
        // mame 1 zonu
        let options = CKFetchRecordZoneChangesOptions()
        
        // nastaveni puvodniho tokenu
        options.previousServerChangeToken = UserDefaults.standard.serverChangeToken
        // ...
        optsDict[ckZoneID] = options
        
        // alokace NSOperation
        let operation = CKFetchRecordZoneChangesOperation(recordZoneIDs: [ckZoneID], optionsByRecordZoneID: optsDict)
        
        // hlaseni o CKRecord, ktere mam lokalne ulozit
        // (jsou pro me nove)
        operation.recordChangedBlock = { record in
            //
            recordsToSave.append(record)
        }
        
        // zaznamy pro smazani z lokalni kopie
        operation.recordWithIDWasDeletedBlock = { recordID, _ in
            //
            recordIDsToDelete.append(recordID)
        }
        
        // komplexni zprava o dokonceni fetche
        operation.recordZoneFetchCompletionBlock = {
            // id zony (zname); moreComing
            zone, newToken, _, more, error in
            
            //
            self.__diffFetchInProgress = false
            
            //
            print("DF ended")
            
            //
            if let _nt = newToken, error == nil {
                // ulozeni noveho tokenu
                UserDefaults.standard.serverChangeToken = _nt
                
                //
                print("DF ended - newtok")
                
                // ...
                DispatchQueue.main.async {
                    //
                    self.processUpdates(toSave: recordsToSave, toDel: recordIDsToDelete)
                    
                    //
                    whenDone()
                }
            }
        }
        //
        ckDB.add(operation)
    }
}
