//
//  dataManagement.swift
//  cviko-tv
//
//  Created by Martin Hruby on 17.04.18.
//  Copyright © 2018 Martin Hruby. All rights reserved.
//

import Foundation
import CloudKit
import UIKit
import CoreData

//
class DataMAN: NSObject {
    //
    static let shared = DataMAN()
    
    //
    lazy var APP: AppDelegate = {
        //
        return UIApplication.shared.delegate as! AppDelegate
    }()
    
    //
    lazy var MOC: NSManagedObjectContext = {
        //
        return APP.persistentContainer.viewContext
    }()
    
    //
    lazy var ckDB = CKContainer.default().privateCloudDatabase
    lazy var ckZone = CKRecordZone(zoneName: "working")
    lazy var ckZoneID = self.ckZone.zoneID
    
    //
    var __syncLocked: Bool = false
    var __diffFetchInProgress: Bool = false
    
    //
    func localSAVE() {
        //
        DispatchQueue.main.async {
            //
            self.APP.saveContext()
        }
    }
    
    //
    func startup() {
        //
        if UserDefaults.standard.firstRUN {
            //
            startupZones()
            startupCloudSubscriptions()
            
            //
            UserDefaults.standard.firstRUN = false
        }
        
        //
        DispatchQueue.global().async {
            //
            self.doDiffFetches {
                //
            }
        }
    }
    
    //
    func startupZones() {
        //
        ckDB.save(self.ckZone) { _, error in
            //
            if let _error = error {
                //
                print("Erorr vytvareni zony: \(_error)")
            } else {
                //
                print("Zone OK")
            }
        }
    }
    
    //
    func startupMOCFollower() {
        //
        NotificationCenter.default.addObserver(self, selector: #selector(mocEvent(_:)), name: Notification.Name.NSManagedObjectContextObjectsDidChange, object: MOC)
    }
    
    //
    func startupCloudSubscriptions() {
        // pojmenovani subscription. Lze dodat NSPredicate
        let cks = CKDatabaseSubscription(subscriptionID: "my-updates")
        
        //
        let notifInfo = CKNotificationInfo()
        
        // varianta notifikace bez UI (ticha notifikace)
        notifInfo.shouldSendContentAvailable = true
        cks.notificationInfo = notifInfo
        
        //
        let operation = CKModifySubscriptionsOperation(subscriptionsToSave: [cks],
                                                       subscriptionIDsToDelete: [])
        //
        operation.modifySubscriptionsCompletionBlock = { _, _, error in
            //
            print("Subscriptions: Error \(error)")
        }
        
        //
        ckDB.add(operation)
    }
}


extension CKRecord {
    // zakodovani soucasneho stavu CKRecord do Data pro
    // ucely synchronizace s CK (ukladam metadata)
    func __primEncode() -> Data {
        //
        let archivedData = NSMutableData()
        let archiver = NSKeyedArchiver(forWritingWith: archivedData)
        archiver.requiresSecureCoding = true
        
        // CKrecord zakoduje svoje metadata
        encodeSystemFields(with: archiver)
        archiver.finishEncoding()
        
        //
        return archivedData as Data
    }
}

extension SyncedNote {
    class func allRecords(modifOnly: Bool) -> [SyncedNote]? {
        //
        return SyncedNote.allRecords(entityName: "SyncedNote", modifOnly: modifOnly) 
    }
}

extension CKSynced {
    //
    class func allRecords<T:NSManagedObject>(entityName: String, modifOnly: Bool) -> [T]? {
        //
        let _req = NSFetchRequest<T>(entityName: entityName)
        
        //
        if modifOnly {
            //
            _req.predicate = NSPredicate(format: "ckmodified=true")
        }
        
        //
        guard let _res = try? DataMAN.shared.MOC.fetch(_req) else {
            //
            return nil
        }
        
        //
        return _res
    }
    
    //
    func generateCK() -> CKRecord? {
        //
        if let _ckenco = self.ckencoded {
            //
            let unarchiver = NSKeyedUnarchiver(forReadingWith: _ckenco)
            unarchiver.requiresSecureCoding = true
            
            // rekonstrukce CKRecord z kodu
            return CKRecord(coder: unarchiver)
        } else {
            //
            let ck = CKRecord(recordType: entity.name!, zoneID: DataMAN.shared.ckZoneID)
            
            //
            return ck
        }
    }
    
    //
    func wasSyncedToCloud(withCKRecord ck: CKRecord) {
        //
        self.ckencoded = ck.__primEncode()
        self.ckmodified = false
        self.ckrecordname = ck.recordID.recordName
    }
}


